import { F, contains, curryN, cond, when, pipe, nthArg, is, T, always, __ } from 'ramda'
import isPatch from './isPatch'

const valueType = cond([
  [ contains(__, ['move', 'copy']), always('from') ],
  [ contains(__, ['add', 'replace', 'test']), always('value') ],
  [ T, F ]
])

/**
 * An utility constructor for JSON Patches given the operator,
 * path and value
 *
 * ** Sig: String -> String -> A -> Object **
 *
 * @function
 * @implements curry
 * @memberof jsonPatch
 * @memberof lib
 * @param {string} operation - A JSON Patch operation (see https://tools.ietf.org/html/rfc6902)
 * @param {string} pointer - A JSON Pointer (see http://tools.ietf.org/html/rfc6901)
 * @param {object} value|from path - A value for [add, replace, test] and a FROM path for [move, copy]
 * @return {object} jsonPatch - A valid json patch
 * @example
 *
 * lib.createPatch('add', '/foo/bar', 123) // { op: 'add', path: '/foo/bar', value: 123 }
 *
 */
const createPatch = curryN(2, (op, jsonPath, value) => {

  let patch = {
    op: op,
    path: jsonPath
  }

  if (valueType(op) !== false) {
    patch[valueType(op)] = value
  }

  let result = isPatch(patch)

  if (true !== result) {
    throw `The provided arguments: op: ${op}, path: ${jsonPath}, value: ${value} are not valid to create a JSON Patch\n Encountered errors: ${result.join('\n')}\n`
  }

  return patch
})

export default createPatch
